/// @file       main.c
/// @copyright  Copyright (c) 2014 Toradex AG
/// @author     $ guc $
/// @date       $ 2014-03-24 $
/// @brief      This is a demo code to show animation on graphic LCD using multiple images. The image file used should be
///             a 128x64, monochrome .bmp file.

#include <windows.h>
#include <stdlib.h>
#include "gpioLib.h"
#include "CoProcLib.h"
#include "GLCD12864Lib.h"
#include "GlcdToradexLogo.h"

//******************************************************************************
/// Content of the .bmp file is converted into GLCD format and then displayed
/// @param[in]    inputArray    Pointer to image buffer of size 1024 bytes, received from .bmp file.
void BmpToGlcd(BYTE *inputArray)
{
    DWORD glcdPage = 0;
    DWORD bitNumber = 0;
    DWORD tempCount1 = 0;
    DWORD tempCount2 = 0;
    DWORD tempCount3 = 0;
    DWORD glcdColumn = 0;
    DWORD columnNumber = 0;
    DWORD glcdImageArray = 0;
    INT tempRasterScanNumber = 0;
    BYTE glcdData[1024] = {0};                       ///< Buffer of pixel values in GLCD format
    BYTE tempBitsBuffer[8192] = {0};                 ///< Buffer to store unpacked bits from inputArray (1024 bytes x 8 Bits)

    /// Converting data from file, into bits (Raster scanning)
    for (tempRasterScanNumber = 896; tempRasterScanNumber >= 0; tempRasterScanNumber = tempRasterScanNumber - 128)
    {
		for (tempCount1 = 0; tempCount1 < 128; tempCount1++)
		{
			for (bitNumber = 0; bitNumber < 8; bitNumber++)
			{
				tempBitsBuffer[tempCount3] = (inputArray[tempRasterScanNumber + tempCount1] & ((0x80)>>bitNumber));
				if (tempBitsBuffer[tempCount3] > 0)
					tempBitsBuffer[tempCount3] = 1;
				else
					tempBitsBuffer[tempCount3] = 0;
				tempCount3++;
			}
		}
	}

    /// Converting bits buffer into GLCD format
    tempCount1 = 0;
    for (tempCount1 = 0; tempCount1 < 1024; tempCount1++)
    {
		if ((tempCount1%128 == 0) && (tempCount1 != 0))
		{
			tempCount2 = tempCount2 + 1024;
			columnNumber = 0;
		}
		for (bitNumber = 0; bitNumber < 8; bitNumber++)
		{  
			glcdData[tempCount1] = (glcdData[tempCount1] | ((tempBitsBuffer[(tempCount2+columnNumber+(bitNumber*128))]&0x01) << (7- bitNumber)));
		}
		columnNumber++;
	}

    /// Displaying image on GLCD
    GlcdSetPage(GLCD_PAGE1, 0);                  ///< Initial page of GLCD
    for (glcdPage = 0; glcdPage < 8; glcdPage++)
    {
        for (glcdColumn = 0; glcdColumn < 128; glcdColumn++)
        {
            GlcdWriteData(glcdData[glcdImageArray++]);
        }
        GlcdSetPage(GLCD_PAGE1+glcdPage+1, 0);   ///< Increment GLCD page
    }
}

//******************************************************************************
/// Obtains the pixel values present in the bitmap file
/// @param[in]    path      Pointer to the string having path of input image file
/// @param[out]   content   Buffer of pixel values of image, size 1024 bytes
/// @retval       TRUE      Success
///               FALSE     Failure.
BOOL OpenBmpFile(char *path, BYTE *content)
{
    FILE *filePointer;
    DWORD bytesRead = 0;
    DWORD dataOffset = 0;
    BYTE info[50] = {0};
    DWORD imageInformation = 0;

    fopen_s(&filePointer, path, "r");                ///< Open image file
    if (filePointer == NULL)
    {
		printf("File cannot be opened because of error: %d", GetLastError());
		return FALSE;
    }

    /// File is correct bitmap image or not
    fseek(filePointer, 0, 0);
    bytesRead = fread(info, sizeof(BYTE), 2, filePointer);
    if (!(strcmp(info, "BM") == 0))
    {
		printf("File is not correct bitmap Image  %s \n", info);
        return FALSE;
    }

    /// Getting offset of bitmap data from the beginning of the file
    fseek(filePointer, 10, 0);
    bytesRead = fread(info, sizeof(BYTE), 4, filePointer);
    dataOffset =  (int)info[0];

    /// Horizontal width of image
    fseek(filePointer, 18, 0);
    bytesRead = fread(info, sizeof(BYTE), 4, filePointer);
    imageInformation =  (int)info[0];
    if (imageInformation != 128)
    {
		printf("Horizontal width is %d,\nPlease enter image of Horizontal width 128\n", imageInformation);
        return FALSE;
    }

    /// Vertical height of image
    fseek(filePointer, 22, 0);
    bytesRead = fread(info, sizeof(BYTE), 4, filePointer);
    imageInformation =  (int)info[0];
    if (imageInformation != 64)
    {
        printf("Vertical height is %d,\nPlease enter image of Vertical height 64\n", imageInformation);
        return FALSE;
    }

    /// Number of layers in bitmap image 
    fseek(filePointer, 26, 0);
    bytesRead = fread(info, sizeof(BYTE), 2, filePointer);
    imageInformation =  (int)info[0];
    if (imageInformation != 1)
    {
        printf("Number of Layers in bitmap Image is %d,\nEnter black and white Image \n", imageInformation);
        return FALSE;
    }

    /// Bits per pixel (BPP) in bitmap image
    fseek(filePointer, 28, 0);
    bytesRead = fread(info, sizeof(BYTE), 2, filePointer);
    imageInformation =  (int)info[0];
    if (imageInformation != 1)
    {
        printf("BPP of bitmap Image is %d,\nPlease enter Monochrome image \n", imageInformation);
        return FALSE;
    }

    /// Type of compression done on bitmap image
    fseek(filePointer, 30, 0);
    bytesRead = fread(info, sizeof(BYTE), 4, filePointer);
    imageInformation =  (int)info[0];
    if (imageInformation != 0)
    {
        printf("Compression Type in bitmap Image is %d,\nEnter uncompressed bitmap image  \n",imageInformation);
        return FALSE;
    }

    /// Getting pixel values into buffer
    fseek(filePointer, dataOffset, 0);
    bytesRead = fread(content, sizeof(BYTE), 1024, filePointer);

    fclose(filePointer);
    return TRUE;
}

//******************************************************************************
int wmain(void)
{
    DWORD imageCount = 0;
    DWORD tempCount = 0;
    CHAR finalPath[60] = {0};
    CHAR toRepeat = {0};
    BYTE fileContent[1024] = {1};
   
    GlcdBackLight(TRUE);
    Sleep(1000);
    GlcdInit();
    GlcdClear();
    GlcdBackLight(TRUE);
    printf("Displaying Image:\n");

    while (TRUE)
    {
		toRepeat = 0;
        
        if (!OpenBmpFile("\\Program Files\\glcd_imagedemo\\images\\photo.bmp", fileContent))    ///< Function call to obtain content of .bmp image
        {
            GlcdBackLight(FALSE);
            GlcdDeInit();
            getchar();
            exit(1);
        }
        BmpToGlcd(fileContent);                                                     ///< Function call to display obtained image content on GLCD
        Sleep(3000);

        /// Display animation on GLCD
        for (tempCount = 1; tempCount <= 6; tempCount++)
        {
            for (imageCount = 1; imageCount <= 11; imageCount++)
            {
                sprintf_s(finalPath, 60, "\\Program Files\\glcd_imagedemo\\images\\horsepicture_%d.bmp", imageCount);
                if(!OpenBmpFile(finalPath, fileContent))                            ///< Function call to obtain content of .bmp image
                {
                    GlcdBackLight(FALSE);
                    GlcdDeInit();
                    getchar();
                    exit(1);
                }
                BmpToGlcd(fileContent);                                             ///< Function call to display obtained image content on GLCD
                Sleep(100);
            }
        }
        if (!OpenBmpFile("\\Program Files\\glcd_imagedemo\\images\\horsepicture_12.bmp", fileContent))   ///< Function call to obtain content of .bmp image
        {
            GlcdBackLight(FALSE);
            GlcdDeInit();
            getchar();
            exit(1);
        }
        BmpToGlcd(fileContent);                                                     ///< Function call to display obtained image content on GLCD
        Sleep(2000);

        GlcdSetPage(GLCD_PAGE1, 0);
        GlcdToradexLogo();                                                          ///< Display Toradex Logo on GLCD
        printf("Press Enter to continue and 'e' to exit\n");
        scanf_s("%c", &toRepeat);
        if (toRepeat == 'e')  break;
    }
    GlcdBackLight(FALSE);
    GlcdDeInit();
    return TRUE;
}

 